import { Client, errors } from "socialvoid";
import { NextRouter } from "next/router";
import { ProviderContext } from "notistack";
import { client } from "../client";
import { runOnClient } from "./client";
import { SessionExpired } from "socialvoid/dist/errors";

export const dispatch = (
  func: (client: Client) => Promise<void> | void,
  router: NextRouter,
  snackbar: ProviderContext,
  options?: {
    requireToBeAuthenticated?: boolean;
    requireToBeNotAuthenticated?: boolean;
  }
) => {
  if (
    options?.requireToBeAuthenticated &&
    options.requireToBeNotAuthenticated
  ) {
    throw new Error("Invalid options");
  }

  runOnClient(async () => {
    try {
      if (options?.requireToBeAuthenticated) {
        if (!client.sessionExists) {
          router.push("/signin");
          return;
        }
      } else if (options?.requireToBeNotAuthenticated) {
        if (client.sessionExists) {
          router.push("/");
          return;
        }
      }

      await func(client);
    } catch (err) {
      if (err instanceof errors.SocialvoidError) {
        if (
          err instanceof errors.SessionExpired ||
          err instanceof errors.SessionNotFound ||
          err instanceof errors.BadSessionChallengeAnswer ||
          err instanceof errors.InvalidSessionIdentification
        ) {
          client.deleteSession();
          snackbar.enqueueSnackbar("Session expired.", {
            variant: "error",
            preventDuplicate: true,
          });
          return;
        }

        snackbar.enqueueSnackbar(
          err.errorMessage.endsWith(".")
            ? err.errorMessage
            : err.errorMessage + ".",
          {
            variant: "error",
            preventDuplicate: true,
          }
        );
      } else {
        console.error(err);
        snackbar.enqueueSnackbar("An unexpected error occurred.", {
          variant: "error",
          preventDuplicate: true,
        });
      }
    }
  });
};
